"""
/**********************************************************************
FILE     : $RCSfile: html2safehtml.py,v $
PURPOSE  : html2safehtml with enhanced functionality and speed
NOTES    : based on html2safehtml by Chris Withers
AUTHOR   : Chris Withers (original), Niels Mache
COPYRIGHT: 
LICENCE  : MIT License
DATE     : 24SEP2001
REVISION : $Revision: 1.6 $
VERSION  : $Id: html2safehtml.py,v 1.6 2001/10/29 22:05:24 mache Exp $ (Author: $Author: mache $)

struktur AG            Phone: +49 711 8966560
Junghansstr. 5         Fax:   +49 711 89665610
70469 Stuttgart        email: info@struktur.de
GERMANY 

http://www.struktur.de
http://www.strukturag.com

**********************************************************************/

Test program for html2safehtml

Copyright (c) 2001 Chris Withers
This Software is released under the MIT License:
http://www.opensource.org/licenses/mit-license.html
See license.txt for more details.

"""


__version__='$Revision: 1.6 $'[11:-2]  

import re
from HTMLParser import HTMLParser,HTMLParseError,piclose, charref, entityref
from string import lower,find

class HTML2SafeHTML(HTMLParser):
	can_close   = ['li','p','dd','dt','option']
	never_close = ['br','wbr','hr','input','isindex','base','meta']

	def __init__(self, valid_tags = {}):
		HTMLParser.__init__(self)
		self.valid_tags = valid_tags
		self.result = ''
		self.openTags = []

	def end_tag(self,tag):
		self.result = self.result + '</%s>' % tag

			
	def handle_data(self, data):
		if data:
			if len(self.openTags) == 0 or len(self.openTags) >= 1 and self.openTags[len(self.openTags) - 1] != 'pre':
				# if not preformatted substitute white space and quote quotation mark
#				dx = re.sub('[\n\r ]+', ' ', data)
				dx = re.sub('\s+', ' ', data)    # substitute multiple white space characters
				dx = re.sub('"', '&quot;', dx)   # quote quotes in data section
#				print 'strip: ', dx
				self.result = self.result + dx
			else:
#				print 'STRING: ', data
				self.result = self.result + data


	def handle_charref(self, name):
		self.result = self.result + "&#%s;" % name

	from htmlentitydefs import entitydefs # our entity defs list to use
	
	def handle_entityref(self, name):
		# this quotes non-standard entities
		if self.entitydefs.has_key(name): 
			amp = '&'
		else:
			amp = '&amp;'

		self.result = self.result + "%s%s;" % (amp, name)


	def handle_starttag(self, tag, attrs):
		""" Delete all tags except for legal ones """
		if self.valid_tags.has_key(tag):
			attribs = self.valid_tags[tag]
			if (len(attribs) == 0):  # ignore all attributes
				self.result = self.result + '<' + tag + '>'
			else:  # pick some ttributes
				self.result = self.result + '<' + tag

				for k, v in attrs:
					if attribs.has_key(k):
						# pick the attribute
						if attribs[k] is None:
							# remove attribute
							pass
						elif attribs[k] == '*':
							# take attribute value. Output in qoutes
							self.result = self.result + ' %s="%s"' % (k, v)  # output attribute
							# modify attribute value
						elif attribs[k][0] == 'r':
							# substitute (replace) attribute value
							self.result = self.result + ' %s="%s"' % (k, re.sub(attribs[k][1], attribs[k][2], v))
						elif attribs[k][0] == 's':
							# search attribute value
							try:
								# XXX Hack: optimize this by precompiling regex
								value = re.search(attribs[k][1], v).group(0)
							except:
								value='""'
							self.result = self.result + ' %s="%s"' % (k, value)
						elif attribs[k][0] == 'd':
							# define the attribute value
							self.result = self.result + ' %s="%s"' % (k, attribs[k][1])
						else:
							pass  # nothing to do
					elif attribs.has_key('*'):
					# get attributes unconditional
						self.result = self.result + ' %s="%s"' % (k, v)  # output attribute
					elif lower(k[0:2]) != 'on' and lower(v[0:10]) != 'javascript':
						# use double quotes to quote the attribute parameters
						self.result = self.result + ' %s="%s"' % (k, v)

				self.result = self.result + '>'

			if tag not in self.never_close:
				self.openTags.append(tag)

	def handle_endtag(self, tag):
		try:            
			while tag != self.openTags[-1] and self.openTags[-1] in self.can_close:
				self.openTags.pop()
			
			if tag==self.openTags[-1]:
				self.end_tag(self.openTags.pop())                

		except IndexError:
			pass

	def cleanup(self):
		""" Append missing closing tags """
		while self.openTags:
			tag = self.openTags.pop()
			if tag not in self.can_close:
				self.end_tag(tag)

	def parse_starttag(self,i):
		try:
			return HTMLParser.parse_starttag(self,i)
		except HTMLParseError:
			try:
				return piclose.search(self.rawdata,i).end()
			except AttributeError:
				return -1
		
	def parse_endtag(self,i):
		try:
			return HTMLParser.parse_endtag(self,i)
		except HTMLParseError:
			try:
				return piclose.search(self.rawdata,i).end()
			except:
				return -1

	def goahead(self,end):
		# fix incomplete entity and char refs        
		rawdata = self.rawdata
		
		i = 0
		n = len(rawdata)
		newdata=''
		
		while i < n:
			j = find(rawdata,'&',i)
			if j==-1:
				break
			newdata = newdata + rawdata[i:j]
			if not (charref.match(rawdata, j) or entityref.match(rawdata, j)):
				newdata = newdata + '&amp;'
			else:
				newdata = newdata + '&'
			i = j+1
			
		self.rawdata = newdata + rawdata[i:]

		# do normal parsing
		try:
			return HTMLParser.goahead(self,end)
		except HTMLParseError:
			pass

