"""
/**********************************************************************
FILE     : $RCSfile: myparse.py,v $
PURPOSE  : test program for html2safehtml
NOTES    : 
AUTHOR   : Niels Mache
COPYRIGHT: 
LICENCE  : MIT License
DATE     : 24SEP2001
REVISION : $Revision: 1.4 $
VERSION  : $Id: myparse.py,v 1.4 2001/10/29 22:05:13 mache Exp $ (Author: $Author: mache $)

struktur AG            Phone: +49 711 8966560
Junghansstr. 5         Fax:   +49 711 89665610
70469 Stuttgart        email: info@struktur.de
GERMANY 

http://www.struktur.de
http://www.strukturag.com

**********************************************************************/

Test program for html2safehtml
This Software is released under the MIT License:
http://www.opensource.org/licenses/mit-license.html
See license.txt for more details.

There is no manual at this point. Please have a look into the code and the Mini-Usage below.

Usage:

HTML2SafeHTML parser is used to cleanup HTML code.
The following code fragment explains the usage of HTML2SafeHTML.
The cleanup behaviour of HTML2SafeHTML parser is controlled by the argument of HTML2SafeHTML (valid_tags).

	parser = HTML2SafeHTML(valid_tags)
	parser.feed(data)
	parser.close()
	parser.cleanup()
	print parser.result

Arguments:
valid tags: A map of maps containing processing instructions. Elements that are not
enumerated in th valid_tags map will be removed from input.

   Get element, but not the attributes (attributes of the given tag will be deleted).
   The given elements remain unchanged.
     <tagname> : { '*' : None }  

   Get element and attibutes. The given elements and attributes are unchanged.
      <tagname> : {}

   Get element and the given attributes and values. Delete all other unnamed attributes.
   Attribute values of named attributes will not be changed.
     <tagname> : { <attribname1> : None , <attribname2> : None , <attribname2> : None }  

   Get element and the given attributes with modifications. Currently, matching [m],
   substitution [s] and definition [d] of attribute values are supported.
     <tagname> : { <attribname1> : ['[m|s|d]', <regular expression1> [, <substitution1>]]}


	valid_tags = {'td' : { '*' : None },    # get element but not the attributes
				  'tr' : { '*' : None },    # get element but not the attributes
				  'table' : { '*' : None }, # get element but not the attributes
				  'tbody' : { '*' : None }, # get element but not the attributes
				  'span' : {},              # get element and all attributes
				  'font' : {},              # get element and all attributes
				  'b' : {},                 # get element and all attributes
				  'i' : {},                 # get element and all attributes
				  'br' : {},                # get element and all attributes
				  'p' : {},                 # get element and all attributes
				  'a' : { '*' : None },     # remove all attributes in anchors
				  'pre' : {},               # get element and all attributes
				  'ul' : {},                # get element and all attributes
				  'li' : {}                 # get element and all attributes
				}


"""


__version__='$Revision: 1.4 $'[11:-2]  

from HTMLParser import HTMLParser,HTMLParseError,piclose, charref, entityref
from string import lower,find
from html2safehtml import HTML2SafeHTML


def test(args = None):
	import sys, formatter

	if not args:
		args = sys.argv[1:]

	silent = args and args[0] == '-s'
	if silent:
		del args[0]

	if args:
		file = args[0]
	else:
		file = 'test.html'

	if file == '-':
		f = sys.stdin
	else:
		try:
			f = open(file, 'r')
		except IOError, msg:
			print file, ":", msg
			sys.exit(1)

	data = f.read()

	if f is not sys.stdin:
		f.close()

	"""
The valid tags is a map of maps:

Examples:
   get tag with all attributes:
     <tagname> : { '*' : None }  
or
   get tag but no attibutes
      <tagname> : {}
or
   get tag and the given attributes with their values.
   delete all other attributes
     <tagname> : { <attribname1> : None , <attribname2> : None , <attribname2> : None }  
or
   get tag and the given attributes with modifications. Matching, substitution and
   definition of attribute values is supported.
     <tagname> : { <attribname1> : ['[m|s|d]', <regular expression1> [, <substitution1>]]}
	"""

	valid_tags = {'td' : { '*' : None },    # delete all mso attributes
				  'tr' : { '*' : None },    # all attributes in tables
				  'table' : { '*' : None }, # all attributes in tables
				  'tbody' : { '*' : None }, # all attributes in tables
#				  'span' : {},
#				  'font' : {},
				  'b' : {},
				  'i' : {},
				  'br' : {},
				  'p' : {},
				  'a' : { '*' : None },   # all attributes in anchors
				  'pre' : {},
				  'ul' : {},
				  'li' : {}
				}

	parser = HTML2SafeHTML(valid_tags)
	parser.feed(data)
	parser.close()
	parser.cleanup()
	print parser.result


if __name__ == '__main__':
	test()
